# snakecase [![NPM version](https://img.shields.io/npm/v/snakecase.svg?style=flat)](https://www.npmjs.com/package/snakecase) [![NPM monthly downloads](https://img.shields.io/npm/dm/snakecase.svg?style=flat)](https://npmjs.org/package/snakecase) [![NPM total downloads](https://img.shields.io/npm/dt/snakecase.svg?style=flat)](https://npmjs.org/package/snakecase)

> Convert a string to snakecase. Similar to kebab-case but uses underscores instead of dashes.

Please consider following this project's author, [Jon Schlinkert](https://github.com/jonschlinkert), and consider starring the project to show your :heart: and support.

## Install

Install with [npm](https://www.npmjs.com/) :

```sh
$ npm install --save snakecase
```

## Prerequisites

Requires [Node.js](https://nodejs.org/en/) version >= 14.

## Usage

```js
const snakecase = require('snakecase');
// or
const { snakecase } = require('snakecase');

console.log(snakecase('a')); //=> 'a'
console.log(snakecase('foo bar baz')); //=> 'foo_bar_baz'
console.log(snakecase('  foo bar baz  ')); //=> 'foo_bar_baz'
console.log(snakecase('foo_bar-baz')); //=> 'foo_bar_baz'
console.log(snakecase('foo.bar.baz')); //=> 'foo_bar_baz'
console.log(snakecase('foo/bar/baz')); //=> 'foo_bar_baz'
console.log(snakecase('foo[bar)baz')); //=> 'foo_bar_baz'
console.log(snakecase('#foo+bar*baz')); //=> 'foo_bar_baz'
console.log(snakecase('$foo~bar`baz')); //=> 'foo_bar_baz'
console.log(snakecase('_foo_bar-baz-')); //=> 'foo_bar_baz'
console.log(snakecase('foo 2 bar 5 baz')); //=> 'foo_2_bar_5_baz'
console.log(snakecase('foo2bar5baz')); //=> 'foo2bar5baz'
```

## Options

### .preserveConsecutiveUppercase

**Type**: `boolean`

**Default**: `false`

This library attemps to sensibly match sequences of uppercase characters in a way that mirrors real-world usage. For example, the following is default behavior:

```js
console.log(snakecase('JSONStringify')); //=> 'json_stringify
console.log(snakecase('TheIRSIsMean')); //=> 'the_irs_is_mean
```

However, if you wish to preserve upper case character sequences, you may pass an options object with the `preserveConsecutiveUppercase` option set to `true`:

```js
console.log(snakecase('JSONStringify', { preserveConsecutiveUppercase: true })); //=> jsons_tringify
console.log(snakecase('TheIRSIsMean', { preserveConsecutiveUppercase: true })); //=> the_irsi_s_mean
```

### .uppercase

**Type**: `boolean`

**Default**: `false`

Convert the output string to upper case.

```js
console.log(snakecase('fooBarBaz', { uppercase: true })); //=> 'FOO_BAR_BAZ'
console.log(snakecase('FooBarBaz', { uppercase: true })); //=> 'FOO_BAR_BAZ'
```

## Options

All options are passed to [sindresorhus/camelcase](https://github.com/sindresorhus/camelcase), please visit that project to learn about all available options.

## About

<details>
<summary><strong>Contributing</strong></summary>

Pull requests and stars are always welcome. For bugs and feature requests, [please create an issue](../../issues/new).

</details>

<details>
<summary><strong>Running Tests</strong></summary>

Running and reviewing unit tests is a great way to get familiarized with a library and its API. You can install dependencies and run tests with the following command:

```sh
$ npm install && npm test
```

</details>

<details>
<summary><strong>Building docs</strong></summary>

_(This project's readme.md is generated by [verb](https://github.com/verbose/verb-generate-readme), please don't edit the readme directly. Any changes to the readme must be made in the [.verb.md](.verb.md) readme template.)_

To generate the readme, run the following command:

```sh
$ npm install -g verbose/verb#dev verb-generate-readme && verb
```

</details>

### Related projects

Other awesome string libs you might like:

* [pascalcase](https://www.npmjs.com/package/pascalcase): Convert a string to pascal case (upper camelcase). | [homepage](https://github.com/jonschlinkert/pascalcase "Convert a string to pascal case (upper camelcase).")
* [romanize](https://www.npmjs.com/package/romanize): Convert numbers to roman numerals (useful for books, outlines, documentation, slide decks, etc) | [homepage](https://github.com/jonschlinkert/romanize "Convert numbers to roman numerals (useful for books, outlines, documentation, slide decks, etc)")
* [word-wrap](https://www.npmjs.com/package/word-wrap): Wrap words to a specified length. | [homepage](https://github.com/jonschlinkert/word-wrap "Wrap words to a specified length.")
* [wordcount](https://www.npmjs.com/package/wordcount): Count the words in a string. Support for english, CJK and Cyrillic. | [homepage](https://github.com/jonschlinkert/wordcount "Count the words in a string. Support for english, CJK and Cyrillic.")

### Author

**Jon Schlinkert**

* [GitHub Profile](https://github.com/jonschlinkert)
* [Twitter Profile](https://twitter.com/jonschlinkert)
* [LinkedIn Profile](https://linkedin.com/in/jonschlinkert)

### License

Copyright © 2022, [Jon Schlinkert](https://github.com/jonschlinkert).
Released under the [MIT License](LICENSE).

***

_This file was generated by [verb-generate-readme](https://github.com/verbose/verb-generate-readme), v0.8.0, on June 21, 2022._